/*
Toradex AG

This library allows an easy access to the GPIOs
*/


#ifndef __GPIOLIB_H__
#define __GPIOLIB_H__

#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif


//Version Info
#define GPIO_VER_MAJ     2
#define GPIO_VER_MIN     1
#define GPIO_VER_BUILD   1

#define DIR_IN	            0
#define DIR_OUT	            1

#define GPIO_EDGE_RISING 1		
#define EDGE_RISING		    1
#define GPIO_EDGE_FALLING 2
#define EDGE_FALLING	    2
#define GPIO_EDGE_BOTH 3
#define EDGE_BOTH           3

#define MFP_PULLUP          (MFP_PULL_SEL | MFP_PULLUP_EN)
#define MFP_PULLDOWN        (MFP_PULL_SEL | MFP_PULLDOWN_EN)
#define MFP_NOPULL          MFP_PULL_SEL
#define MFP_EDGE_DISABLE    MFP_EDGE_CLEAR

#define MFP_PULL_SEL        0x8000
#define MFP_PULLUP_EN       0x4000
#define MFP_PULLDOWN_EN     0x2000
#define MFP_SLEEP_SEL       0x0200
#define MFP_SLEEP_DATA      0x0100
#define MFP_SLEEP_OE_N      0x0080
#define MFP_EDGE_CLEAR      0x0040
#define MFP_EDGE_FALL_EN    0x0020
#define MFP_EDGE_RISE_EN    0x0010

typedef struct {
	BYTE inst1; //:  8; // instance 1
	BYTE inst2; //:  8; // instance 2
  //DWORD res   : 16; // reserved
} PIN_INSTANCE;



/**************************************
Tegra Specific Defines
**************************************/
//Converts a Tegra GPIO Name in a GPIONr.
//x is Port Name y is bit nr,
//e.g TEGRA_GPIONUM('w',0) is GPIO W4
#define TEGRA_GPIONUM(x,y) ((((x)-'a')*8)+y)

//Define Max lenght of the Group name
#define TEGRA_MAX_GROUPNAME 6
//Define Max number of GPIOGroups
#define TEGRA_MAX_GROUPNUM 115

#define TEGRA_NOPULL	0x00
#define TEGRA_PULLDOWN	0x01
#define TEGRA_PULLUP	0x02





void InitGPIOLib();
BOOL InitGPIOLibEx(void* pRegAdress);
void DeInitGPIOLib();

/**************************************************
Tegra Specific Functions
**************************************************/

/**
Set the Tristate settign of a GPIO
Please note this affect the whole pin group!
@param		gpioNr			GPIONr
@param		pullState (0: nothing, 1:pullup, 2:pulldown)
@retval		TRUE  success
*/
BOOL TegraSetTristatePinGroup(DWORD gpioNum, BOOL tristate);

/**
Get the Tristate settign of a GPIO
@param		gpioNr			GPIONr
@retval		1:Tristated 0:Active
*/
DWORD TegraGetTristatePinGroup(DWORD gpioNum);

/**
Set the Pullup/down settign of a GPIO
Please note this affect the whole pin group!
@param		gpioNr			GPIONr
@param		pullState (0: nothing, 1:pullup, 2:pulldown)
@retval		TRUE  success
*/
BOOL TegraSetPullStatePinGroup(DWORD gpioNum, DWORD pullState);

/**
Get the Pullup/down settign of a GPIO
@param		gpioNr			GPIONr
@retval		Pull status (0: nothing, 1:pullup, 2:pulldown)
*/
DWORD TegraGetPullStatePinGroup(DWORD gpioNum);

/**
Get the Pin Group Name of a GPIO
@param		gpioNr			GPIONr
@param		[out]*groupName	Name of the Pin group max TEGRA_MAX_GROUPNAME characters.
@retval		TRUE  success
*/
BOOL TegraGetPinGroupName(DWORD gpioNum, char * groupName);







/**************************************************
General Functions
**************************************************/


/***************************************************
pinNum: SODIMM Pin Number
return GPIO Direction
****************************************************/
DWORD GetPinDir(DWORD pinNum);

/***************************************************
gpioNum: GPIO Number
return GPIO Direction
****************************************************/
DWORD GetGPIODir(DWORD gpioNum);

/***************************************************
gpioNum: GPIO Number
dirOut: GPIO Direction [true:Output  false:Input]
****************************************************/
void SetPinDir(DWORD gpioNum, BOOL dirOut);

/***************************************************
gpioNum: GPIO Number
dirOut: GPIO Direction [true:Output  false:Input]
****************************************************/
void SetGPIODir(DWORD gpioNum, BOOL dirOut);

/***************************************************
gpioNum: GPIO Number
return	Altrnativ Funktion Nr.
		On the Tegra GPIO function does have AltFn -1
****************************************************/
DWORD GetGPIOAltFn(DWORD gpioNum);



/***************************************************
pinNum: SODIMM Pin Number
return	Altrnativ Funktion Nr.
		On the Tegra GPIO function does have AltFn -1
****************************************************/
DWORD GetPinAltFn(DWORD pinNum);

/***************************************************
pinNum: SODIMM Pin Number
altFn:	Alternativ Funktion
		-1 on does always mean GPIO Function. 
		(You need to use -1 on the Tegra if you like to set a pin as GPIO)
dirOut: GPIO Direction [true:Output  false:Input]
return: TRUE sucsessfull FALSE: faild 
****************************************************/
BOOL SetPinAltFn(DWORD pinNum, DWORD altFn, BOOL dirOut);

/***************************************************
gpioNum: GPIO Number
altFn:	Alternativ Funktion
		-1 on does always mean GPIO Function. 
		(You need to use -1 on the Tegra if you like to set a pin as GPIO)
dirOut: GPIO Direction [true:Output  false:Input]
****************************************************/
void SetGPIOAltFn(DWORD gpioNum, DWORD altFn, BOOL dirOut);


DWORD GetGPIOMFPReg(DWORD gpioNum);
void SetGPIOMFPReg(DWORD gpioNum, DWORD mfpRegVal);


/***************************************************
pinNum: Pin Number
return Pin Level
****************************************************/
BOOL GetPinLevel(DWORD pinNum);

/***************************************************
gpioNum: GPIO Number
return GPIO Level
****************************************************/
BOOL GetGPIOLevel(DWORD gpioNum);

/***************************************************
pinNum: SODIMM Pin. Number
val: [0: Clear   Others: Set]
return: TRUE sucsessfull FALSE: faild 
****************************************************/
BOOL SetPinLevel(DWORD pinNum, DWORD val);

/***************************************************
gpioNum: GPIO Number
val: [0: Clear   Others: Set]
****************************************************/
void SetGPIOLevel(DWORD gpioNum, DWORD val);

/***************************************************
gpioNum: GPIO Number
return: edge [EDGE_RISING:rising edge EDGE_RISING:rising edge Others:Set nothing]
****************************************************/
DWORD GetGPIOEdgeDetect(DWORD gpioNum);

/***************************************************
gpioNum: GPIO Number
edge: [EDGE_RISING:rising edge EDGE_RISING:rising edge Others:Set nothing]
****************************************************/
void SetGPIOEdgeDetect(DWORD gpioNum, DWORD edge);

/***************************************************
gpioNum: GPIO Number
return: The EdgeStatus
****************************************************/
BOOL GetGPIOEdgeStatus(DWORD gpioNum);

/***************************************************
gpioNum: GPIO Number
****************************************************/
void ClearGPIOEdgeStatus(DWORD gpioNum);


/***************************************************
pin: Pin Number (SODIMM pin or extesion connector pin)
extensionConnector: FALSE:SODIMM pin   TRUE:Extension connector
[out] gpio: gpio Nummer with second instance
return: TRUE sucsessfull FALSE: faild 
****************************************************/
BOOL GetGPIOFromPin(DWORD pin, BOOL extensionConnector,PIN_INSTANCE *gpio);


/***************************************************
gpioNum:	GPIO Number
return: Alternative Function with normal GPIO functionality
        Normal case 0;
        No GPIO Function: -1;
****************************************************/
DWORD GetGPIOFunctionalityAltFn(DWORD gpioNum);

/*************************************
Description:      Returns the library Version

Parameter:
                  Out-Parameter:
                  - pVerMaj:          Returns the major version number. Set this parameter to NULL if not required.
                  - pVerMin:          Returns the minor version number. Set this parameter to NULL if not required.
                  - pBuild:           Returns the build number.         Set this parameter to NULL if not required.
*************************************/
void GPIOGetLibVersion(DWORD *pVerMaj, DWORD *pVerMin, DWORD *pBuild);


#ifdef __cplusplus
}
#endif


#endif